from tkinter import *
import math
#
# ■相対描画のためのRelativeDrawクラス定義
#　  
#   変数名 = RelativeDraw(canvas, startX, startY,scaleX, scaleY, lineWidth,color)
#
#       canvas          : キャンバス
#       startX, startY  : 表示開始位置
#       scaleX, scaleY  : 表示スケール(上下または左右を逆転するときは負の値で指定)
#       lineWidth       : 描画する直線の太さ
#       color           : カラーを文字列で指定(例 : 'red', '#00FFFF')
#
class RelativeDraw:
    #■ コンストラクタ
    def __init__(self, canvas,startX, startY,scaleX, scaleY, lineWidth,color):
        self.canvas=canvas
        self.X0=startX
        self.Y0=startY
        self.SX=scaleX
        self.SY=scaleY
        self.lineWidth=lineWidth
        self.color=color
        self.goto(0,0)
        
    #■ XY座標設定
    def goto(self,x,y):
        self.x=x
        self.y=y
        
    #■ スクリーン座標への変換
    def screenX(self, X):
        return self.X0 + self.SX *  X
    def screenY(self, Y):
        return self.Y0 + self.SY *  Y

class TurtleGraph(RelativeDraw):
    #■ コンストラクタ
    def __init__(self, canvas,startX, startY,scaleX, scaleY, lineWidth,color):
        RelativeDraw.__init__(self,canvas,startX, startY,scaleX, scaleY,
                              lineWidth,color)
        self.setAng(0)
    #■ 角度設定
    def setAng(self,th):
        self.th=th

    #■ 向きの変更(左回転は負の値で指定)
    def turn(self,th):
        self.th=self.th+th

    #■ 前に進む(後退は負の値で指定)
    def forward(self,L):
        TH=self.th*math.pi/180
        X=self.x+L*math.cos(TH)
        Y=self.y+L*math.sin(TH)
        self.canvas.create_line(self.screenX(self.x), self.screenY(self.y),
                                self.screenX(X)     , self.screenY(Y)     ,
                                width = self.lineWidth, fill = self.color)
        self.x=X
        self.y=Y

        
#■相対位置まで曲線を描くクラス
class RelativeCurve(RelativeDraw):
    def draw(self,X,Y):
        NX=self.x+X
        NY=self.y+Y
        self.canvas.create_line(self.screenX(self.x), self.screenY(self.y),
                                self.screenX(NX)     , self.screenY(NY)     ,
                                width = self.lineWidth, fill = self.color)
        self.x=NX
        self.y=NY
        
#■シエルピンスキ曲線のクラス定義(主処理以外はRelativeCurve,RelativeDrawクラスからの継承)
class SierpinskiCurve(RelativeCurve):
    #移動量の設定
    def setH(self,order):
        self.ID=order
        H = 6
        S = 2000
        S = S / H
        for i in range(2, order+1):
            H=H*2+2
        self.H=S/H
    #ヒルベルト曲線を描く
    def curve(self, order):
        self.setH(order)
        self.goto(0,0)
        self.URD()
        self.draw( self.H,  self.H)
        self.LUR()
        self.draw(-self.H,  self.H)
        self.DLU()
        self.draw(-self.H, -self.H)
        self.RDL()
        self.draw( self.H, -self.H)
    #右→下→左
    def RDL(self):
        if self.ID>0:
            self.ID=self.ID-1
            self.RDL()
            self.draw( self.H, -self.H)
            self.URD()
            self.draw( 0, -2*self.H)
            self.DLU()
            self.draw(-self.H, -self.H)
            self.RDL()
            self.ID=self.ID+1
    #下→左→上        
    def DLU(self):
        if self.ID>0:
            self.ID=self.ID-1
            self.DLU()
            self.draw(-self.H, -self.H)
            self.RDL()
            self.draw(-2*self.H, 0)
            self.LUR()
            self.draw(-self.H,  self.H)
            self.DLU()
            self.ID=self.ID+1
    #左→上→右
    def LUR(self):
        if self.ID>0:
            self.ID=self.ID-1
            self.LUR()
            self.draw(-self.H, self.H)
            self.DLU()
            self.draw(0, 2*self.H)
            self.URD()
            self.draw( self.H, self.H)
            self.LUR()
            self.ID=self.ID+1
    #上→右→下
    def URD(self):
        if self.ID>0:
            self.ID=self.ID-1
            self.URD()
            self.draw(self.H, self.H)
            self.LUR()
            self.draw(2*self.H, 0)
            self.RDL()
            self.draw(self.H, -self.H)
            self.URD()
            self.ID=self.ID+1
    
#■ tkinter，Canvas共通処理
def initTk(Title):
    tk=Tk()
    tk.title(Title)
    return tk
def initCanvas(W,H):
    canvas=Canvas(tk,width=W,height=H)
    canvas.pack()
    return canvas

#■　メイン処理
tk=initTk("Sierpinski Curve used tkinter")
canvas=initCanvas(500,500)
S=SierpinskiCurve(canvas, 50, 400, 1, -1, 2, 'blue')
S.curve(5)

            
        
